package cn.skcks.docking.gb28181.wvp.service.gb28181;

import cn.skcks.docking.gb28181.core.sip.gb28181.sdp.GB28181Description;
import cn.skcks.docking.gb28181.core.sip.gb28181.sdp.MediaSdpHelper;
import cn.skcks.docking.gb28181.core.sip.gb28181.sdp.StreamMode;
import cn.skcks.docking.gb28181.media.config.ZlmMediaConfig;
import cn.skcks.docking.gb28181.media.dto.rtp.GetRtpInfoResp;
import cn.skcks.docking.gb28181.media.dto.rtp.OpenRtpServer;
import cn.skcks.docking.gb28181.media.dto.rtp.OpenRtpServerResp;
import cn.skcks.docking.gb28181.media.dto.status.ResponseStatus;
import cn.skcks.docking.gb28181.media.proxy.ZlmMediaService;
import cn.skcks.docking.gb28181.service.ssrc.SsrcService;
import cn.skcks.docking.gb28181.wvp.config.ProxySipConfig;
import cn.skcks.docking.gb28181.wvp.orm.mybatis.dynamic.model.WvpProxyDevice;
import cn.skcks.docking.gb28181.wvp.service.device.DeviceService;
import gov.nist.javax.sdp.fields.TimeField;
import lombok.RequiredArgsConstructor;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.sdp.Connection;
import javax.sdp.SdpFactory;
import javax.sdp.TimeDescription;
import javax.sip.ListeningPoint;
import java.util.Date;
import java.util.List;
import java.util.Optional;
import java.util.Vector;

@Slf4j
@Service
@RequiredArgsConstructor
public class Gb28181DownloadService {
    private final ZlmMediaService zlmMediaService;
    private final ZlmMediaConfig zlmMediaConfig;
    private final SsrcService ssrcService;
    private final DeviceService deviceService;
    private final ProxySipConfig proxySipConfig;

    private String videoUrl(String streamId) {
        return StringUtils.joinWith("/", zlmMediaConfig.getUrl(), "rtp", streamId + ".live.flv");
    }

    private int openRtpServer(String streamId, int streamMode) {
        GetRtpInfoResp rtpInfo = zlmMediaService.getRtpInfo(streamId);
        if (rtpInfo.getExist()) {
            log.warn("流 {} 已存在", streamId);
            return 0;
        }

        OpenRtpServer openRtpServer = new OpenRtpServer();
        openRtpServer.setPort(0);
        openRtpServer.setStreamId(streamId);
        openRtpServer.setTcpMode(streamMode);
        OpenRtpServerResp openRtpServerResp = zlmMediaService.openRtpServer(openRtpServer);
        log.info("openRtpServerResp => {}", openRtpServerResp);
        if (!openRtpServerResp.getCode().equals(ResponseStatus.Success)) {
            log.error("{}", openRtpServerResp.getCode().getMsg());
            return -1;
        }
        return openRtpServerResp.getPort();
    }

    public void download(String deviceCode, Date startTime, Date endTime){
        List<WvpProxyDevice> deviceByDeviceCode = deviceService.getDeviceByDeviceCode(deviceCode);
        if (deviceByDeviceCode.isEmpty()) {
            log.info("未能找到 设备编码 为 {} 的设备",deviceCode);
        } else {
            WvpProxyDevice device = deviceByDeviceCode.get(0);
            download(device.getGbDeviceId(), device.getGbDeviceChannelId(), startTime, endTime);
        }
    }

    @SneakyThrows
    public void download(String gbDeviceId, String channel,Date startTime, Date endTime){
        Optional<WvpProxyDevice> deviceByGbDeviceIdAndChannel = deviceService.getDeviceByGbDeviceIdAndChannel(gbDeviceId, channel);
        if (deviceByGbDeviceIdAndChannel.isEmpty()) {
            log.info("未能找到 编码 {}, 通道 {} 的设备", gbDeviceId, channel);
            return ;
        }

        long start = startTime.toInstant().getEpochSecond();
        long end = endTime.toInstant().getEpochSecond();
        String streamId = MediaSdpHelper.getStreamId(gbDeviceId, channel, String.valueOf(start), String.valueOf(end));
        int streamMode = proxySipConfig.getTransport().equalsIgnoreCase(ListeningPoint.UDP) ? 0 : 1;
        String ip = zlmMediaConfig.getIp();
        int port = openRtpServer(streamId, streamMode);
        String ssrc = ssrcService.getPlaySsrc();
        TimeField timeField = new TimeField();
        timeField.setStartTime(start);
        timeField.setStopTime(end);
        TimeDescription timeDescription = SdpFactory.getInstance().createTimeDescription(timeField);
        GB28181Description gb28181Description = MediaSdpHelper.playback(gbDeviceId, channel, Connection.IP4, ip, port, ssrc, StreamMode.of(ListeningPoint.UDP), startTime, endTime);
        gb28181Description.setSessionName(SdpFactory.getInstance().createSessionName(MediaSdpHelper.Action.DOWNLOAD.getAction()));
        gb28181Description.setTimeDescriptions(new Vector<>(){{add(timeDescription);}});
        log.debug("{}", gb28181Description);
    }

}
